//#include "base.h"
#include <QMessageBox>
#include <QTextStream>
#include <QFileInfo>
#include <QDir>
#include <QInputDialog>
#include <QDebug>

int which_array(QString &name, QList<Array_>& arrays)
{
int result=-1;
size_t n=arrays.size();
for (size_t i=0; i<n; i++)
	if (name==arrays.at(i).name) result=i;
return result;
}

/*
void print_parameters(gsimFD info)
{
    for (size_t i=0; i<info.parameterFiles.size(); i++) {
    cout<<"File:"<<info.parameterFiles.at(i).toAscii().data()<<endl;
    QHash<QString, QString> lPar= info.allParameters.at(i);
    QHashIterator<QString, QString> iter(lPar);
    while (iter.hasNext()) {
        iter.next();
		cout<<iter.key().toAscii().data()<<" : "<<iter.value().toAscii().data()<<endl;
	}
    }
}
*/

bool isreadable(const char* fname)
{
  FILE* fd=NULL;
  fd=fopen(fname,"rt");
  if (fd!=NULL) {
    fclose(fd);
    return true;
  }
  else
    return false;
}

bool is_spinsight(const char* fname)
{/* Looking for files "data" and "acq" */
     char testname[256];
     strcpy(testname,fname);
     if (strlen(testname)<4) return false;
     char* p=testname+strlen(testname)-4;
     if (strcmp(p,"data")) {
       return false;
     }
     if (!isreadable(testname))
       return false;
     strcpy(p,"acq");
     return isreadable(testname);
}

bool is_xwinnmr(const char* fname)
{/* Looking for files "fid" or "ser" and "acqu" */
     char testname[256];
     strcpy(testname,fname);
     if (strlen(testname)<3) return false;
     char* p=testname+strlen(testname)-3;
     if (strcmp(p,"fid") && strcmp(p,"ser")) {
       return false;
     }
     if (!isreadable(testname))
       return false;
     strcpy(p,"acqu");
     return isreadable(testname);
}

bool is_xwinnmrproc(const char* fname)
{/* Looking for files "fid" or "ser" and "acqu" */
	QString fn=QString(fname);
	if (!QFile::exists(fn))
		return false;
	if (!fn.endsWith("1r"))
		return false;

	fn.chop(2);
	fn+="1i";
	if (!QFile::exists(fn))
		return false;
	fn.chop(2);
	fn+="proc";
	if (!QFile::exists(fn))
		return false;
	return true;
}


bool is_vnmr(const char* fname)
{/* Looking for files "fid" and "procpar" */
     char testname[256];
     strcpy(testname,fname);
     if (strlen(testname)<3) return false;
     char* p=testname+strlen(testname)-3;
     if (strcmp(p,"fid")) {
       return false;
     }
     if (!isreadable(testname))
       return false;
     strcpy(p,"procpar");
     return isreadable(testname);
}

bool is_simpson(const char* fname)
{       
	bool result=false;
	QFile file(fname);
        if (!file.open(QIODevice::ReadOnly | QIODevice::Text))
            return false;
        QTextStream in(&file);
	QString line = QString(in.readLine());
	if (!line.contains(QString("SIMP")))
		result=false;
	else
		result=true;
	file.close();
	return result;
}

bool is_spinevolution(const char* fname)
{       
	bool result=true;
	QFile file(fname);
        if (!file.open(QIODevice::ReadOnly | QIODevice::Text))
            return false;
        QTextStream in(&file);
	QString line = QString(in.readLine());
	QStringList values=line.split(" ",QString::SkipEmptyParts);
	if (values.size()<2)
		result=false; //each string contains at least 1 number
	bool ok;
	for (size_t i=0; i<values.size(); i++) {
		values.at(i).toDouble(&ok); //all entries should be numbers
		if (!ok) result=false;
	}
	file.close();
	return result;
}

int read_spinsight_parfile(QString fname, QHash<QString, QString> &pars) //don't forget to use parameterFiles separately
{
	QFile file(fname);
        if (!file.open(QIODevice::ReadOnly | QIODevice::Text))
            return 1; //file doesn't exist
        QTextStream in(&file);
        while (!in.atEnd()) {
            QString line = in.readLine();
//	    cout<<line.toAscii().data()<<endl;
            if (line.contains("=")) {
		QStringList buf=line.split("=", QString::SkipEmptyParts);
		if (buf.size()<2)
			pars[buf.at(0)]=QString("");
		else {
			if (buf.at(1).endsWith("s"))
				buf[1].chop(1);
			if (buf.at(1).endsWith("u") || buf.at(1).endsWith("m"))
				buf[1].chop(1);
			pars[buf.at(0)]=buf.at(1);
		}
	    }
        }
	file.close();
	return 0;
}

int read_spinsight(cmatrix &a, gsimFD &par, QList<Array_> &arrays, const char* fname)
/*function for reading of SPINSIGHT 1D FIDs and spectra. FID stores in "data" file in binary format.
4 bytes per each (real or imag) point in big-endian format (as integer). Spectra - the same but as float. First half of file stores the real part, second half -imaginary
adopted partly from MatNMR of J. vanBeek. Thanks!*/

{
  FILE* pFile;
  unsigned char* datat;
  char parfname[256];
  par.ni=1; //default value
  par.sw1=1.0;
  int dataformat=FD_TYPE_FID;
  double freq[4];
  int ch1=0, ch2=0;
  double rmp1=0.5, rmp2=0.5;
  double rmv1=0.0, rmv2=0.0;
  int rmvunits1=2, rmvunits2=2;

  //Reading acq file
  strcpy(parfname,fname);
  strcpy(parfname+strlen(parfname)-4,"acq");

//load parameters
 
  par.parameterFiles<<"acq";
  QHash<QString, QString> lPar; //loaded parameters
  int fault=read_spinsight_parfile(parfname, lPar);
  if (fault) {
	throw Failed("read_spinsight: acq file doesn't exist");
  }
  par.allParameters.append(lPar);

//analyse parameters
if (lPar.contains("ch1"))
    ch1=lPar.value("ch1").toInt();
if (lPar.contains("ch2"))
    ch2=lPar.value("ch2").toInt();
if (lPar.contains("sf1"))
    freq[0]=lPar.value("sf1").toDouble();
if (lPar.contains("sf2"))
    freq[1]=lPar.value("sf2").toDouble();
if (lPar.contains("sf3"))
    freq[2]=lPar.value("sf3").toDouble();
if (lPar.contains("sf4"))
    freq[3]=lPar.value("sf4").toDouble();
if (lPar.contains("ppfn"))
    par.pulprog=lPar.value("ppfn");
if (lPar.contains("sw"))
    par.sw=lPar.value("sw").toDouble()*1e3;
if (lPar.contains("sw2"))
    par.sw1=lPar.value("sw2").toDouble()*1e3;
if (lPar.contains("al2"))
    par.ni=lPar.value("al2").toInt();
if (lPar.contains("al"))
    par.np=lPar.value("al").toInt();


//find arrays:
    QHashIterator<QString, QString> iter(lPar);
    while (iter.hasNext()) {
        iter.next();
      if (iter.key().contains('[') && iter.key().contains(']'))
	{
//	   cout<<"Found "<<iter.key().toAscii().data()<<" with value "<<iter.value().toDouble()<<endl;
	   QString s=iter.key();
	   QStringList ls=s.split("[", QString::SkipEmptyParts);
	   int k=which_array(ls[0], arrays); //ls[0] - array name
	   if (k<0) {  //array doesn't exist
			Array_ a(ls[0]);
			arrays.append(a);
			k=arrays.size()-1;
		}
		int index=ls[1].remove("]").toInt();
//		cout<<"Index:"<<index<<endl;
		while(arrays[k].data.size()<=index)
			arrays[k].data.push_back(0.0);
		arrays[k].data(index)=iter.value().toDouble();
	}
  	}

//Determine frequency in both dimensions
if (ch1>4 || ch2>4)
	throw Failed("read_spinsight: wrong channel in acq files");
par.sfrq=freq[ch1-1]; //direct frequency is a frequencty of channel 1
par.sfrq1=freq[ch2-1]; //direct frequency is a frequencty of channel 1

//The same thing for 'proc' file

  strcpy(parfname+strlen(parfname)-3,"proc");

//load parameters

  par.parameterFiles<<"proc";
  lPar.clear(); //loaded parameters
//  cout<<parfname<<"\n";
  fault=read_spinsight_parfile(parfname, lPar);
  if (fault)
	throw Failed("read_spinsight: proc file doesn't exist");
  par.allParameters.append(lPar);


//analyse parameters
if (lPar.contains("com"))
    par.title=lPar["com"].remove('\n');
if (lPar.contains("datatype")) {
	double value=lPar.value("datatype").toDouble();
	if (!(value==1.0 || value==0.0))
		throw Failed("read_spinsight: datatype in 'proc' file is not 0 or 1");
      if (value==1.0) dataformat=FD_TYPE_SPE;
      else dataformat=FD_TYPE_FID;
}

if (lPar.contains("domain1")) {
	double value=lPar.value("domain1").toDouble();
	if (!(value==1.0 || value==0.0))
		throw Failed("read_spinsight: domain1 in 'proc' file is not 0 or 1");
      if (value==1.0) par.type=FD_TYPE_SPE;
      else par.type=FD_TYPE_FID;
    }

if (lPar.contains("domain2")) {
	double value=lPar.value("domain2").toDouble();
	if (!(value==1.0 || value==0.0)) 
			throw Failed("read_spinsight: domain2 in 'proc' file is not 0 or 1");
      if (value==1.0) par.type1=FD_TYPE_SPE;
      else par.type1=FD_TYPE_FID;
    }

if (lPar.contains("current_size1"))
      par.np=lPar.value("current_size1").toInt();
if (lPar.contains("current_size2"))
      par.ni=lPar.value("current_size2").toInt();
if (lPar.contains("rmp1"))
      rmp1=lPar.value("rmp1").toDouble();
if (lPar.contains("rmp2"))
      rmp2=lPar.value("rmp2").toDouble();
if (lPar.contains("rmv1"))
      rmv1=lPar.value("rmv1").toDouble();
if (lPar.contains("rmv2"))
      rmv2=lPar.value("rmv2").toDouble();
if (lPar.contains("rmvunits1"))
      rmvunits1=lPar.value("rmvunits1").toInt();
if (lPar.contains("rmvunits2"))
      rmvunits2=lPar.value("rmvunits2").toInt();

switch (rmvunits1) { //direct dimension
	case 1://ppm
		rmv1*=par.sfrq; //convert to Hz
		par.ref=rmv1+(par.sfrq-rmp1)*1e6;
		break;
	case 2://Hz
		par.ref=rmv1+(par.sfrq1-rmp1)*1e6;
		break;
}

switch (rmvunits2) { //indirect dimension
	case 1:
		rmv2*=par.sfrq1; //convert to kHz
		par.ref1=rmv2+(par.sfrq1-rmp2)*1e6;
		break;
	case 2:
		par.ref1=rmv2+(par.sfrq1-rmp2)*1e6;
		break;
}

  strcpy(parfname+strlen(parfname)-4,"proc_setup");
  par.parameterFiles<<"proc_setup";
  lPar.clear(); //loaded parameters
//  cout<<parfname<<"\n";
  fault=read_spinsight_parfile(parfname, lPar);
  if (fault)
	throw Failed("read_spinsight: proc_setup file doesn't exist");
  par.allParameters.append(lPar);

//open data file
  pFile=fopen(fname, "rb");
  if (pFile==0) 
      throw Failed("read_spinsight: File 'data' is not found");
// obtain file size and reset to start 
  fseek(pFile, 0, SEEK_END);
  const long lSize = ftell(pFile);
  rewind(pFile);
  if (lSize & 7) {
    fclose(pFile);
	throw Failed("read_spinsight: file length is not multiple of 8. Corrupt?");
  }

   if ((par.np*par.ni)!=lSize/8) {
	fclose(pFile);
	throw Failed("read_spinsight: file length is not match parameters. Corrupt?");
	}
// allocate memory to contain the whole file. 
  datat = (unsigned char*) malloc(lSize);
  if (datat==NULL) {
	fclose(pFile);
	throw Failed("read_spinsight: Cannot allocate memory for data from 'data' file");
	}
// copy the file into the buffer.
  fread(datat, 1, lSize, pFile);
  fclose(pFile);
// Change byte order if mashine is little-endian
  if (!ambigendian()) {
    unsigned char* ptr=datat;
    size_t i=0;
    for (i=lSize/4;i--;ptr+=4) {
      swap_bytes(ptr,ptr+3);
      swap_bytes(ptr+1,ptr+2);
    }
  }

  const int len=lSize/8;
  List<complex> templist;
  a.clear();
  if (dataformat==FD_TYPE_FID)  {
    int32_t* asint=(int32_t*)datat;
    complex tmp(0.0,0.0);
    for (int i=0; i<len; i++) {
      tmp.re=asint[i];
      tmp.im=asint[i+len];
      templist.push_back(tmp);
    }
  }
  else {
    if (sizeof(float)!=4)
	throw Failed("read_spinsight: 'float' data type on this machine is incompatible with GSim: only raw FID could be loaded");
    float* asfloat=(float*)datat;
    complex tmp(0.0,0.0);
    for (int i=0; i<len; i++) {
      tmp.re=asfloat[i];
      tmp.im=asfloat[i+len];
      templist.push_back(tmp);
    }
   }
  a.create(par.ni, par.np);
  for (size_t i=par.ni; i--;)
  for (size_t j=par.np; j--;)
	a(i,j)=templist(j+par.np*i);
  free(datat);
  return 0;
}

int read_xwinnnmr(cmatrix &a, gsimFD &par, QList<Array_> &arrays, const char* fname)
{
	read_xwinnmr_arrays(arrays, fname);
	int result= read_xwinnmr(a,par,fname);
	return result;
}

void read_xwinnmr_array(Array_ &array, QFile* file)
{
	 while (!file->atEnd()) {
            QString line = QString(file->readLine());
            line.remove(QChar(' '));
	    line.remove(QChar('\n'));
	    double multiplier=1.0;
	    if (line.at(line.size()-1)==QChar('s')) {
		multiplier=1.0e3;
		line.truncate(line.size()-1);
		}
	    if (line.at(line.size()-1)==QChar('m')) {
		multiplier=1.0;
		line.truncate(line.size()-1);
		}
	    if (line.at(line.size()-1)==QChar('u')) {
		multiplier=1.0e-3;
		line.truncate(line.size()-1);
		}
//		cout<<line.toAscii().data()<<";"<<endl;
	    bool convOK=true;
	    array.data.push_back(line.toDouble(&convOK)*multiplier);
	    if (!convOK) {
		file->close();
		throw ("read_bruker: array file is corrupted");
		}
//		QMessageBox::critical(0, "read_bruker", "Line "+line+" can not be read");
        }
}

void read_xwinnmr_arrays(QList<Array_> &arrays, const char* fname)
{
	QStringList names;
	names<<"valist"<<"vclist"<<"vdlist"<<"vplist"<<"vtlist"<<"fqlist"<<"masrlist";
	for (size_t i=names.size(); i--;) {
		QString arfname=fname;
		arfname.chop(3);
		arfname+=names.at(i);
		qDebug()<<"Looking for"<<arfname;
		QFile va(arfname);
        	if (va.open(QIODevice::ReadOnly | QIODevice::Text)) {
			Array_ ar(names.at(i).toAscii().data());
			read_xwinnmr_array(ar, &va);
			va.close();
			arrays.push_back(ar);
		}
	}
}

int read_xwinnmr_parfile(QString fname, QHash<QString, QString> &pars) //don't forget to use parameterFiles separately
{
	QFile file(fname);
        if (!file.open(QIODevice::ReadOnly | QIODevice::Text))
            return 1; //file doesn't exist
        QTextStream in(&file);
        while (!in.atEnd()) {
            QString line = in.readLine();
            if (line.contains("=")) {
		QStringList buf=line.split("=", QString::SkipEmptyParts);
		buf[0].remove("##$");
		if (buf.size()<2)
			pars[buf.at(0)]=QString("");
		else
			pars[buf.at(0)]=buf.at(1);
	    }
        }
	file.close();
	return 0;
}


int read_xwinnmr(cmatrix &a, gsimFD &par, const char* fname)
/*function for reading of Bruker XWIN-NMR FIDs. FID stores in "fid" or "ser" file in binary format.
4 bytes per each (real or imag) point in big-endian format (as integer). Real and imaginary part is interlacing.
adopted partly from MatNMR of J. vanBeek. Thanks!*/
{
  FILE* pFile;
//  FILE* acqFile;

  unsigned char* datat;
  char parfname[256];
  par.ni=1; //default value
  par.sw1=1.0;
  par.format=FD_TYPE_FID;

//Reading acq file
  strcpy(parfname,fname);
  strcpy(parfname+strlen(parfname)-3,"acqu");
//  cout<<parfname<<endl;
//load parameters
 
  par.parameterFiles<<"acqu";
  QHash<QString, QString> lPar; //loaded parameters
  int fault=read_xwinnmr_parfile(parfname, lPar);
  if (fault) {
	throw Failed("read_bruker: acqu file doesn't exist");
  }
  par.allParameters.append(lPar);

//analysing parameters
if (lPar.contains("SW_h"))
      par.sw=lPar.value("SW_h").toDouble();
if (lPar.contains("SFO1"))
      par.sfrq=lPar.value("SFO1").toDouble();
if (lPar.contains("PULPROG")) {
      par.pulprog=lPar.value("PULPROG");
      par.pulprog.remove('\n');
      par.pulprog.remove('>');
      par.pulprog.remove('<');
}

//The same thing for 'acqu2' file
//Reading acqu2 file
  strcpy(parfname+strlen(parfname)-4,"acqu2");

  par.parameterFiles<<"acqu2";
  lPar.clear(); //loaded parameters
  fault=read_xwinnmr_parfile(parfname, lPar);
if (!fault) {
  par.allParameters.append(lPar);

//analysing parameters
if (lPar.contains("SW_h"))
      par.sw1=lPar.value("SW_h").toDouble();
if (lPar.contains("SFO1"))
      par.sfrq1=lPar.value("SFO1").toDouble();
if (lPar.contains("TD"))
      par.ni=lPar.value("TD").toInt();
}
else
	par.parameterFiles.removeLast();	

//The same thing for 'acqus' file which needed for digital filtered data
//Reading acqus file
  strcpy(parfname+strlen(parfname)-5,"acqus");

  par.parameterFiles<<"acqus";
  lPar.clear(); //loaded parameters
  fault=read_xwinnmr_parfile(parfname, lPar);
//  cout<<parfname<<endl;
if (!fault)
  par.allParameters.append(lPar);
else
  par.parameterFiles.removeLast();

int bytorda=1;
if (lPar.contains("BYTORDA"))
      bytorda=lPar.value("BYTORDA").toInt();

//print_parameters(par);
//trying to read a title from specified file
  strcpy(parfname,fname);
//  QString pfname;

  QFileInfo finfo(parfname); //create File info
  QDir curdir=finfo.absoluteDir(); //take a directory name
   if (curdir.cd("pdata"))  //if possible to move to pdata directory
  {
  QStringList savedproc=curdir.entryList(QDir::Dirs|QDir::NoDotAndDotDot, QDir::Name);
  if (savedproc.size()) {
  QString procdir;
  if (savedproc.size()>1)
  	procdir=QInputDialog::getItem(0, "Select Bruker processed data", "Several processed data are found, please select one", savedproc);
  else
	procdir=savedproc.at(0);
//  strcpy(parfname+strlen(parfname)-5,"pdata/"); //looking for 'title' file for proc no.1
    QString pfname=curdir.absolutePath()+"/"+procdir+"/title";
//  strcat(parfname,procdir.toAscii().data());
//  strcat(parfname,"/title");
  cout<<pfname.toAscii().data()<<endl;

  QFile textFile(pfname);
        if (textFile.open(QIODevice::ReadOnly | QIODevice::Text)) {
		QTextStream in2(&textFile);
		par.title=QString("proc/1: ")+in2.readLine();
		par.title.remove(QChar('\n'));
	}
  textFile.close();
  pfname=curdir.absolutePath()+"/"+procdir+"/proc";
    cout<<"File:"<<pfname.toAscii().data()<<endl;
  par.parameterFiles<<"proc";
  lPar.clear(); //loaded parameters
    fault=read_xwinnmr_parfile(pfname.toAscii().data(), lPar);
//  cout<<parfname<<endl;
if (!fault) {
  par.allParameters.append(lPar);
  cout<<"done\n";}
else
  par.parameterFiles.removeLast();
if (lPar.contains("OFFSET"))
      par.ref=lPar.value("OFFSET").toDouble()*par.sfrq-par.sw/2.0;

  pfname=curdir.absolutePath()+"/"+procdir+"/proc2";
  par.parameterFiles<<"proc2";
  cout<<"File 2:"<<pfname.toAscii().data()<<endl;
  lPar.clear(); 
    fault=read_xwinnmr_parfile(pfname.toAscii().data(), lPar);
//  cout<<parfname<<endl;
if (!fault) {
  par.allParameters.append(lPar);
  cout<<"done again\n";
	}
else
  par.parameterFiles.removeLast();
if (lPar.contains("OFFSET"))
      par.ref1=lPar.value("OFFSET").toDouble()*par.sfrq1-par.sw1/2.0;
  }


  }
//open data file
  pFile=fopen(fname, "rb");
  if (pFile==0)
	throw Failed("read_bruker: 'ser' or 'fid' file is not found");
  /* obtain file size and reset to start */
  fseek(pFile, 0, SEEK_END);
  const long lSize = ftell(pFile);
//  cout<<"Size:"<<lSize<<endl;
//  cout<<"NI:"<<par.ni<<endl;
  rewind(pFile);
  if (lSize & 7) {
    fclose(pFile);
    throw Failed("read_bruker: file length is not multiple of 8. Corrupt?");
  }
   par.np=lSize/8/par.ni;
//   cout<<"NP:"<<par.np<<endl;
   if ((par.np*par.ni)!=lSize/8) {
	fclose(pFile);
	throw Failed("read_bruker: file length is not match parameters. Corrupt?");
	}
  datat = (unsigned char*) malloc(lSize);
  if (datat==NULL) 
	throw Failed("read_bruker: Cannot allocate memory for data from 'fid'/'ser' file");

// copy the file into the buffer. 
  fread(datat, 1, lSize, pFile);
  fclose(pFile);
  if (!ambigendian()==bytorda) {
    unsigned char* ptr=datat;
    size_t i=0;
    for (i=lSize/4;i--;ptr+=4) {
      swap_bytes(ptr,ptr+3);
      swap_bytes(ptr+1,ptr+2);
    }
  }

  const int len=lSize/4; //for bytes per each number
  List<complex> templist;
  a.clear();
    int32_t* asint=(int32_t*)datat;
    complex tmp(0.0,0.0);
    for (int i=0; i<len; i=i+2) {
      tmp.re=asint[i];
      tmp.im=asint[i+1];
      templist.push_back(tmp);
    }
  a.create(par.ni, par.np);
  for (size_t i=par.ni; i--;)
  for (size_t j=par.np; j--;)
	a(i,j)=templist(j+par.np*i);
  free(datat);
  return 0;
}

int read_xwinnmrproc(cmatrix &a, gsimFD &par, QList<Array_> & arrays, const char* fname)
//function for reading of Bruker XWIN-NMR processing data. Real and imaginary parts stored in 1r and 1i files in binary format. 8 bytes per each (real or imag) point in big-endian format (as integer 64). Could have a big-end li-and problem
{
  FILE* pFile;
  FILE* pFile2;
//  FILE* acqFile;

  unsigned char* datat;
  unsigned char* datat2;
  char parfname[256];
  par.ni=1; //default values
  par.sw1=1.0;
  par.type=FD_TYPE_SPE;
  par.format_on_disk=FFORMAT_XWINNMRPROC;

//Reading proc file
  strcpy(parfname,fname);
  strcpy(parfname+strlen(parfname)-2,"proc");
//load parameters
  par.parameterFiles<<"proc";
  QHash<QString, QString> lPar; //loaded parameters
  int fault=read_xwinnmr_parfile(parfname, lPar);
  if (fault) {
	throw Failed("read_bruker: proc file doesn't exist");
  }
  par.allParameters.append(lPar);
//analysing parameters
if (lPar.contains("SF"))
      par.sfrq=lPar.value("SF").toDouble();
if (lPar.contains("SW_p"))
      par.sw=lPar.value("SW_p").toDouble();
if (lPar.contains("OFFSET"))
      par.ref=lPar.value("OFFSET").toDouble()*par.sfrq-par.sw/2.0;

//trying to read first title
  strcpy(parfname,fname); //read title from 'text' file
  strcpy(parfname+strlen(parfname)-4,"title"); //looking for 'title' file for proc no.1
  QFile textFile(parfname);
        if (textFile.open(QIODevice::ReadOnly | QIODevice::Text)) {
		QTextStream in2(&textFile);
		par.title=in2.readLine();
		par.title.remove(QChar('\n'));
	}
  textFile.close();


//open 1r file
  pFile=fopen(fname, "rb");
  if (pFile==0)
	throw Failed("read_bruker: '1r' file is not found");
  /* obtain file size and reset to start */

  char fname2[256];
  strcpy(fname2, fname);
  strcpy(fname2+strlen(fname)-2,"1i");
  pFile2=fopen(fname2, "rb");
  if (pFile2==0)
	throw Failed("read_bruker: '1i' file is not found");


  fseek(pFile, 0, SEEK_END);
  long lSize = ftell(pFile);
//  cout<<"Size:"<<lSize<<endl;
  rewind(pFile);
  if (lSize & 7) {
    fclose(pFile);
    throw Failed("read_bruker: file length is not multiple of 8. Corrupt?");
  }
   par.np=lSize/8;
//   cout<<"NP:"<<par.np<<endl;

  datat = (unsigned char*) malloc(lSize);
  datat2 = (unsigned char*) malloc(lSize);
  if ((datat==NULL) || (datat2==NULL)) 
	throw Failed("read_bruker: Cannot allocate memory for data from '1r'/'1i' file");

// copy the file into the buffer. 
  fread(datat, 1, lSize, pFile);
  fread(datat2, 1, lSize, pFile2);
  fclose(pFile);
  fclose(pFile2);
  if (!ambigendian()) {
    unsigned char* ptr=datat;
    unsigned char* ptr2=datat2;
    size_t i=0;
    for (i=lSize/8;i--;ptr+=8) {
      swap_bytes(ptr,ptr+3);
      swap_bytes(ptr+1,ptr+2);
      swap_bytes(ptr2,ptr2+3);
      swap_bytes(ptr2+1,ptr2+2);
    }
  }

  const int len=lSize/8; //for bytes per each number
  List<complex> templist;
  a.clear();
    int64_t* asint=(int64_t*)datat;
    int64_t* asint2=(int64_t*)datat2;
    complex tmp(0.0,0.0);
    for (int i=0; i<len; i++) {
      tmp.re=asint[i];
      tmp.im=asint2[i];
      templist.push_back(tmp);
    }
  a.create(par.ni, par.np);
  for (size_t i=par.ni; i--;)
  for (size_t j=par.np; j--;)
	a(i,j)=templist(j+par.np*i);
  free(datat);
  return 0;
}

int read_vnmr_parfile(QString fname, QHash<QString, QString> &pars) //don't forget to use parameterFiles separately
{
	QFile file(fname);
        if (!file.open(QIODevice::ReadOnly | QIODevice::Text))
            return 1; //file doesn't exist
        QTextStream in(&file);
        while (!in.atEnd()) {
            QString line = in.readLine();
	    if (line.at(0).isDigit() || line.at(0)==QChar('"'))
		continue;

	    QStringList values=line.split(" ", QString::SkipEmptyParts);
	    line=in.readLine();
	    line.remove('\n');
	    line.remove('"');
	    pars[values.at(0)]=line;
	    //in.readLine();
        }
	file.close();
	return 0;
}


int read_vnmr(cmatrix &a, gsimFD &par, QList<Array_> &arrays, const char* fname)
/*function for reading of Varian VNMR FIDs. FID stores in "fid" file in binary format.
Adopted from MatNMR of J. vanBeek. Thanks!*/
{
    FILE* pFile;

  unsigned char* datat;
  char parfname[256];
  par.ni=1; //default value
  par.sw1=1.0;
  par.format=FD_TYPE_FID;
  QString refpos, refpos1;
  double reffrq=0, reffrq1=0;
  QStringList values;
  QStringList ArrayNames;

//Reading acq file 
  strcpy(parfname,fname);
  strcpy(parfname+strlen(parfname)-3,"procpar"); //looking for 'procpar' file

  par.parameterFiles<<"procpar";
  QHash<QString, QString> lPar; //loaded parameters
  int fault=read_vnmr_parfile(parfname, lPar);
  if (fault) {
	throw Failed("read_vnmr: 'procpar' file doesn't exist");
  }
  par.allParameters.append(lPar);

//analysing parameters
if (lPar.contains("SW_h"))
      par.sw=lPar.value("SW_h").toDouble();

//read arrays
  if (lPar.contains("array")) {
	values=lPar.value("array").split(" ");
	for (int i=1; i<values.size(); i++)
		ArrayNames<<values.at(i);
}

//print_parameters(par);

for (int q=0; q<ArrayNames.size(); q++) 
{
	if (lPar.contains(ArrayNames.at(q))) {
//		cout<<"Array "<<ArrayNames.at(q).toAscii().data()<<endl;
		QString line=lPar.value(ArrayNames.at(q));
		line.remove('\n');
		values=line.split(" ");
		Array_ ar(ArrayNames.at(q));
		for (int i=1; i<=values.at(0).toInt(); i++)
			ar.data.push_back(values.at(i).toDouble());
		arrays.push_back(ar);
	}
}

  if (lPar.contains("sfrq")) {
	values=lPar.value("sfrq").split(" ");
	par.sfrq=values.at(1).toDouble();
}

  if (lPar.contains("dfrq")) {
	values=lPar.value("dfrq").split(" ");
	par.sfrq1=values.at(1).toDouble();
  }

if (!par.sfrq)
	par.sfrq=par.sfrq1;  //Strange but working

  if (lPar.contains("sw")) {
	values=lPar.value("sw").split(" ");
	par.sw=values.at(1).toDouble();
  }

  if (lPar.contains("sw1")) {
	values=lPar.value("sw1").split(" ");
	par.sw1=values.at(1).toDouble();
	if (par.sw1>1.0)
		par.sfrq1=par.sfrq;
  }

 if (lPar.contains("seqfil")) {
	values=lPar.value("seqfil").split(" ");
	par.pulprog=values.at(1);
	par.pulprog.remove('"');
 }

 if (lPar.contains("reffrq")) {
	values=lPar.value("reffrq").split(" ");
    	reffrq=values.at(1).toDouble();
}

 if (lPar.contains("reffrq1")) {
	values=lPar.value("reffrq1").split(" ");
	reffrq1=values.at(1).toDouble();
}

if (lPar.contains("refpos")) {
	values=lPar.value("refpos").split(" ");
	refpos=values.at(1);
}

if (lPar.contains("refpos1")) {
	values=lPar.value("refpos1").split(" ");
	refpos1=values.at(1);
}


if ((!refpos.isEmpty()) && (refpos!="n")){
//	cout<<"Reffrq:"<<reffrq<<" SFRQ: "<<par.sfrq<<endl;
	par.ref=-(reffrq-par.sfrq)*1e6;
}
if ((!refpos1.isEmpty()) && (refpos1!="n"))
	par.ref1=-(reffrq1-par.sfrq1)*1e6;	

  strcpy(parfname,fname); //read title from 'text' file
  strcpy(parfname+strlen(parfname)-3,"text"); //looking for 'text' file
  QFile textFile(parfname);
        if (textFile.open(QIODevice::ReadOnly | QIODevice::Text)) {
		QTextStream in2(&textFile);
		par.title=in2.readLine();
		par.title.remove(QChar('\n'));
	}
	textFile.close(); 
  /*open data file*/
  pFile=fopen(fname, "rb");
  if (pFile==0) {
	throw Failed("read_vnmr: File 'fid' is not found");
//    QMessageBox::critical(0, "read_vnmr","File 'fid' is not found");
//    return 1;
  }
  /* obtain file size and reset to start */
  fseek(pFile, 0, SEEK_END);
  const long lSize = ftell(pFile);
  rewind(pFile);
//  if (lSize & 7) {
//    fclose(pFile);
//    QMessageBox::critical (0, "read_bruker","file length is not multiple of 8. Corrupt?");
//    return 1;
//  }
//   par.ni=1;
//   par.np=lSize/8/par.ni;
//   if ((par.np*par.ni)!=lSize/8){
//	fclose(pFile);
//    	QMessageBox::critical (0, "read_bruker","file length is not match parameters. Corrupt?");
//    	return 1;
//	}

  /* allocate memory to contain the whole file. */
  datat = (unsigned char*) malloc(lSize);
  if (datat==NULL) {
	throw Failed("read_vnmr: Cannot allocate memory for data from 'fid' file");
//    QMessageBox::critical (0, "read_vnmr","Cannot allocate memory for data from 'fid' file");
//    return 1;
  }
  /* copy the file into the buffer. */
  fread(datat, 1, lSize, pFile);
  fclose(pFile);
/* Change byte order if machine is little-endian*/
  if (!ambigendian()) {
    unsigned char* ptr=datat;
    size_t i=0;
    for (i=lSize/4;i--;ptr+=4) {
      swap_bytes(ptr,ptr+3);
      swap_bytes(ptr+1,ptr+2);
    }
  }

//read in the Varian header
    int32_t* asint=(int32_t*)datat;
    int16_t* asint16 = (int16_t*)datat;
    int32_t  nblocks = asint[0];
    int32_t  ntraces = asint[1];
    size_t  np = asint[2];
//    int32_t  ebytes = asint[3];
//    int32_t  tbytes = asint[4];
//    int32_t  bbytes = asint[5];
//    int16_t  vers_id = asint16[12];
    int16_t  status = asint16[13];
    int32_t  nbheaders = asint[7];
    asint=asint+8;

    size_t ni=nblocks*ntraces;
    List<double> onlydata;

/*    if (((status&0x4)!=0x4) && ((status&0x8) != 0x8))
      for (size_t i=0;i<nblocks; i++){
       for (size_t j=0; j<nbheaders; j++) {
        //fread(id, nbheaders*14, 'int16');	//read in the block headers (nbheaders*28 bytes)
	asint=asint+nbheaders*7;
	asint16=(int16_t *)asint;
       }
       for (size_t q=0; q<ntraces*np; q++)	//read in the actual data (ntraces*np)
		onlydata.push_back(*(asint16+q));
	}
*/
//    else if (((status&0x4)==0x4) && ((status&0x8) != 0x8))

      if ((status&0x8) != 0x8)
      for (size_t i=0;i<nblocks; i++){
       for (size_t j=0; j<nbheaders; j++) {
        //fread(id, nbheaders*14, 'int16');	//read in the block headers (nbheaders*28 bytes)
	asint=asint+nbheaders*7;
//	asint16=(int16_t *)asint;
       }
       for (size_t q=0; q<ntraces*np; q++)	//read in the actual data (ntraces*np)
		onlydata.push_back(*(asint+q));
	asint+=ntraces*np;
	}
 
    else 
for (size_t i=0;i<nblocks; i++){
	float* asfloat;
	asfloat=(float *)asint;
       for (size_t j=0; j<nbheaders; j++) {
        //fread(id, nbheaders*14, 'int16');	//read in the block headers (nbheaders*28 bytes)
	asfloat=asfloat+nbheaders*7; //float supposed to have 4 bytes length
       }
       for (size_t q=0; q<ntraces*np; q++)	//read in the actual data (ntraces*np)
		onlydata.push_back(*(asfloat+q));
	asfloat+=ntraces*np;
	}

  List<complex> templist;
  for (size_t i=0; i<onlydata.size()/2; i++)
	templist.push_back(complex(onlydata(2*i),onlydata(2*i+1)));
  np/=2;
  a.create(ni, np);
  for (size_t i=ni; i--;)
  for (size_t j=np; j--;)
	a(i,j)=templist(j+np*i);
  free(datat);
  par.ni=ni;
  par.np=np;
//  par.sw=10000;
  return 0;
}


#include <sys/types.h>
#include <iostream>
#include <fstream>
#include <string>
#include <sys/stat.h>
#include <sys/types.h>
#include <QDir>

#define LINE_MAX_LEN 256

//you want the absolute max value, not the range
double absmax(const BaseList<complex>& a) //use const if input unmodified
  // More flexible to pass row rather than matrix
{
  double t;
  double max=0.0;
  for (size_t i=a.size();i--;) {
    t=fabs(real(a(i)));
    if (t>max)
      max=t;
    t=fabs(imag(a(i)));
    if (t>max)
      max=t;
  }
  return max;
}

//use a reference for 1st argument, otherwise copied
template <class T> void write_out(const BaseList<complex>& data, int sc, ofstream& f)
{
  const size_t size(data.size());
  List<T> out(size*2); //make sure buffer has correct size

  for (size_t i=size;i--;) {
    out(i)= T(sc*real(data(i)));
    out(i+size)= T(sc*imag(data(i)));
  }

  if (!ambigendian()) {
    unsigned char* p=reinterpret_cast<unsigned char*>(out.vector());
    for (size_t i=size*2;i--;p+=4) {
      ::std::swap(p[0],p[3]);
      ::std::swap(p[1],p[2]);
    }
  }
  const char* ptr=reinterpret_cast<char*>(out.vector());
  f.write(ptr,sizeof(T)*size*2);
//   for (k=0; k<size*4; k++)
//     f<<ptr[k];
}

void write_data(const cmatrix& data, const gsimFD& parameters, const char* dir)
{
  QDir Dir(dir);
  if (!Dir.exists()) {
	bool ok=Dir.mkdir(dir);
	if (!ok) qWarning("Can't create directory");
  }

//if (mkdir(dir, 511))
//    throw Failed("write_data: make directory"); //throw exception if create directory failed

  char datafile[LINE_MAX_LEN];
  snprintf(datafile,sizeof(datafile),"%s/data",dir); //snprintf is safer - strange things will happen if input is too long for LINE_MAX_LEN, but it's less likely to cause a disaster
  //cout<<datafile<<endl;
  ofstream filestr;
  filestr.open(datafile,  ofstream::binary | ofstream::out | ofstream::trunc);
  if (!filestr) 
    throw Failed("write_data: open file"); //only check once - if one file works, the others will
  //size_t size=2*data.size();
  //cout << size << "=" << parameters.ni << "*" << parameters.np <<endl;

  if (parameters.type==FD_TYPE_FID) //FID stored as integers
    {
      double ampl=absmax(data.row());
      int sc=1;
      if (ampl) { //check for zero data!
	double rough_sc=1000000/ampl;
	while (rough_sc>10){
	  rough_sc/=10;
	  sc*=10;}
      }
      write_out<int32_t>(data.row(), sc, filestr);
    }
  else
    write_out<float>(data.row(), 1, filestr);
  filestr.close();
}

void write_parameters(gsimFD par, const char * dir, QList<Array_> arrays)
{
  char tmpname[LINE_MAX_LEN];

  snprintf(tmpname,sizeof(tmpname),"%s/simpson.acq",dir);
  ofstream filestr;
  filestr.open(tmpname, ofstream::out | ofstream::trunc);
  if (!filestr)
    throw Failed("write_parameters: open file");
  filestr<<"ch1;ppg ch1;2;-;1;4;0;1;long\n";
  filestr<<"sf1;ch1 spect freq;100.6;MHz;1.0;800.5;7;1;float\n";
  filestr<<"dw;dwell;50;u;0.13;100000;2;1;float\n";
  filestr<<"sw;spectrum width;20;kHz;0.01;7693;3;1;float\n";
  filestr<<"al;acq length;512;-;2;1048576;0;1;long\n";
  if (par.ni>1) {
    filestr<<"ch2;ppg ch2;2;-;1;4;0;1;long\n";
    filestr<<"sf2;ch2 spect freq;100.6;MHz;1.0;800.5;7;1;float\n";
    filestr<<"dw2;dwell 2;50;u;0.13;100000;2;1;float\n";
    filestr<<"sw2;spectrum width 2;20;kHz;0.01;7693;3;1;float\n";
    filestr<<"al2;acq length 2;512;-;2;1048576;0;1;long\n";
  }
// Write Arrays if exists;
   for (size_t i=0; i<arrays.size(); i++) {
	filestr<<arrays.at(i).name.toAscii().data()<<"; array "<<arrays.at(i).name.toAscii().data()<<";1;m;0.000000000000001;9999999999999.9;3;1;float\n";
	}

  filestr.close();

  snprintf(tmpname,sizeof(tmpname),"%s/acq",dir);
  filestr.open(tmpname, ofstream::out | ofstream::trunc);
//  if (!par.pulprog.isEmpty())
//  	filestr<<"ppfn="<<par.pulprog.toAscii().data()<<"\n";
//  else  //This code can make a mess if open a file in Spinsight
	filestr<<"ppfn=gsim\n";
  filestr<<"ch1=1\n";
  if (par.sfrq<0)
  	filestr<<"sf1=125.0\n";
  else
  	filestr<<"sf1="<<par.sfrq/1e6<<"\n";
  filestr<<"dw="<<1/par.sw<<"\n";
  filestr<<"sw="<<par.sw/1000<<"\n";
  filestr<<"al="<<par.np<<"\n";

  if (par.ni>1){
    filestr<<"ch2=2\n";
    if (par.sfrq1<0)
    	filestr<<"sf2=500.0\n";
   else
        filestr<<"sf2="<<par.sfrq1<<"\n";
    filestr<<"dw2="<<1/par.sw1<<"\n";
    filestr<<"sw2="<<par.sw1/1000<<"\n";
    filestr<<"al2="<<par.ni<<"\n";
  }

// Write Arrays if exists;
   for (size_t i=0; i<arrays.size(); i++) {

   filestr<<"array_num_values_"<<arrays.at(i).name.toAscii().data()<<"="<<arrays.at(i).data.size()<<"\n";
   filestr<<"array_dim_"<<arrays.at(i).name.toAscii().data()<<"=2\n";
   for (size_t j=0; j<arrays.at(i).data.size(); j++)
	filestr<<arrays.at(i).name.toAscii().data()<<"["<<j<<"]="<<arrays.at(i).data(j)<<"m\n";
   }

  filestr.close();

  snprintf(tmpname,sizeof(tmpname),"%s/proc",dir);
  filestr.open(tmpname, ofstream::out | ofstream::trunc);

  filestr<<"com="<<par.title.toAscii().data()<<"\n\n";
  filestr<<"datatype=" << ((par.type==FD_TYPE_FID)? 0 : 1) << '\n'; //integer 32-bit/float 32-bit
  filestr<<"domain1="<<((par.type==FD_TYPE_FID)? 0:1)<<'\n';//time/freq
  filestr<<"current_size1="<<par.np<<"\n";
  filestr<<"rmp1="<<par.sfrq<<"\n";//spectrum centre
  filestr<<"rmv1="<<par.ref/1000<<"\n";//ref at 0
  filestr<<"rmvunits1=2\n";//kHz
  if (par.ni>1){
    filestr<<"current_size2="<<par.ni<<"\n";
    filestr<<"domain2="<<((par.type1==FD_TYPE_FID)? 0:1)<<'\n';
    filestr<<"rmp2="<<par.sfrq1<<"\n";
    filestr<<"rmv2="<<par.ref1<<"\n";
    filestr<<"rmvunits2=2\n";
    }
  filestr.close();


  snprintf(tmpname,sizeof(tmpname),"%s/proc_setup",dir);
  filestr.open(tmpname, ofstream::out | ofstream::trunc);
  filestr<<"ft1=1\n";
  if (par.ni>1){
    filestr<<"psort1=1\n";
    filestr<<"ft2=1\n";
    }
  filestr.close();

}

void write_spinsight(const cmatrix& data, gsimFD parameters, const char* dir, QList<Array_> arrays)
{
  if (parameters.ni==0)
    parameters.ni=1;
  write_data(data, parameters, dir);
  write_parameters(parameters, dir, arrays);
}

int read_spinevolution(cmatrix &a, gsimFD &par, const char* fname)
{
	QFile file(fname);
        if (!file.open(QIODevice::ReadOnly | QIODevice::Text))
            return 1;
	QList<QStringList> pts;
	QTextStream in(&file);
	QString buffer;
        while (!in.atEnd()) {
            	buffer=in.readLine();
		pts.append(buffer.split(" ",QString::SkipEmptyParts));
	} //data is loaded
//	cout<<pts.size()<<" read with "<<pts.at(0).size()<<" in each\n";
	file.close();
	par.ni=pts.at(0).size()-1;
//	cout<<"NI:"<<par.ni<<endl;
	par.np=pts.size();
//	cout<<"Np:"<<par.np<<endl;
	a.create(par.ni,par.np,complex(0.0,0.0));
	for (size_t i=0; i<par.ni; i++)
	for (size_t j=0; j<par.np; j++) {
		a(i,j).re=pts.at(j).at(i+1).toDouble();
//		cout<<i<<":"<<j<<endl;
		}
	par.type=FD_TYPE_SPE;
	par.type1=FD_TYPE_SPE;
	par.sw=fabs(pts.at(0).at(0).toDouble()-pts.at(pts.size()-1).at(0).toDouble());
	par.sw1=par.sw;
//	cout<<"sw="<<par.sw<<"\n";
	return 0;
}
